clear all;
close all;

%% Boost Converter - Parameters:
T_S = 1/100000;         % Switching period
L = 0.001;              % Inductance
C = 0.000995;           % Output Capacitor (recalculated for requested voltage ripple)
v_out = 400;            % Output Voltage

R = v_out^2/500;        % Load Resistor


%% Grid data:
V_AC = 230;             % RMS value of grid voltage
f = 50;                 % grid frequency


%% Current Controller - Parameters:
k_pi = 0.0555;
T_ni = 1/(2*pi*5000);


%% Voltage Controller - Design:

% Transfer-Function of plant and current controller
Z2=[0 v_out];           % Numerator of plant
N2=[L 0];               % Denominator of plant
G2=tf(Z2,N2);           % total transfer function

% Parameters of current controller
K_ic = 1737.8;          % Proportional gain => K_ic = k_pi*2*pi*f_z1
f_z1 = 5000;            % Crossing frequency of PI-controller
w_z1 = f_z1*2*pi;       % Crossing angular frequency
Z_ic = K_ic*[1/w_z1 1]; % Numerator of transfer function
N_ic = [1 0];           % Denominator of transfer function
G_ic = tf(Z_ic,N_ic);   % total transfer function

% Open current control loop
Zo = conv(Z2,Z_ic);
No = conv(N2,N_ic);
Go = tf(Zo,No);

% Closed current control loop
Zi = Zo;
Ni = No+Zo;
Gi = tf(Zi,Ni);     

% Transfer Function of R-C-Output
Z_rc = [V_AC*R];
N_rc = [R*C 1];
G_rc = tf(Z_rc,N_rc);

% Transfer Functions of closed current control loop and RC-output
Z_us = conv(Zi,Z_rc);
N_us = conv(Ni,N_rc);
G_us = tf(Z_us,N_us);
figure(20)
hold on;
bode (G_us,{10,1000000});

% Design of the PI-voltage controller
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
f_z2 =  10;              % Crossover frequency => Please choose an appropriate value
k_pu =  0.07;               % Proportional gain =>  Please choose an appropriate value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
w_z2 = f_z2*2*pi;


T_nu = 1/w_z2;
Z_uc = k_pu*[T_nu 1];
N_uc = [T_nu 0];
G_uc = tf(Z_uc,N_uc);
bode(G_uc,{10,1000000});
grid on;

% Open voltage control loop
Zuo = conv(Z_us,Z_uc);
Nuo = conv(N_us,N_uc);
Guo = tf(Zuo,Nuo);

bode(Guo,{10,1000000});
grid on;

legend('Closed Current Control Loop & RC Output', 'PI Voltage Controller', 'Open Voltage Control Loop');

